/*
*    TriggerMan - An Audit Table and Trigger Management Tool for MySQL. 
*    Version 1.0 - April 01, 2013. 
*    Copyright (C) 2013 Impirical Software Engineering  www.impiricalsoftware.com
*    David J. Hutira  djhutira@impiricalsoftware.com
*
*    This program is free software: you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation, either version 3 of the License, or
*    (at your option) any later version.
*
*    This program is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*/
package triggerman;

import java.awt.Color;
import java.awt.Component;
import java.io.*;
import java.sql.*;
import java.util.List;
import javax.swing.*;
import javax.swing.RowSorter.SortKey;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.table.*;
import oreilly.*;

/**
 *
 * @author Dave Hutira
 */
public class TriggerMan extends javax.swing.JFrame {

    static String urlPrefix = "jdbc:mysql://localhost:3306/";
    static String infoSchemaURL = urlPrefix + "information_schema";         

    static String acct; 
    static String pwd; 

    static Connection infoConn;
    static Connection conn;    
    
    static String schemaName; 
    static String auditSchemaName; 
    
    static TableModel nullModel; 
    
    static int[] selectedIndexes = new int[100]; 
    static int[] selectedViewIndexes = new int[100]; 
    static int numSelected; 
    static String selectedTableName; 
    static String selectedAuditTableName; 
    static int[] columnWidths = new int[8]; 
    static boolean widthSaved = false; 
    
    //Table TMan_Options
    static String auditTablePrefix; 
    static String auditTableSuffix; 
    static String triggerNameFormat; 
    static String beforeString; 
    static String afterString; 
    static String deleteString; 
    static String insertString; 
    static String updateString; 
    static String tmpAuditTablePrefix; 
    static boolean multiBDDefault; 
    static boolean multiBIDefault; 
    static boolean multiBUDefault; 
    static boolean multiADDefault; 
    static boolean multiAIDefault; 
    static boolean multiAUDefault; 
    static boolean multiSaveRestoreDefault; 
    static boolean multiRebuildAuditDefault; 
    static String multiStr = "-- MultiSelect --"; 
    static boolean multiBDExists; 
    static boolean multiBIExists; 
    static boolean multiBUExists; 
    static boolean multiADExists; 
    static boolean multiAIExists; 
    static boolean multiAUExists; 
    static boolean multiAuditExists;     
    
    static ListSelectionModel tableSelectionModel; 
    static boolean loadInProgress = false; 
    
    /**
     * Creates new form TriggerManFrame
     */
    public TriggerMan() {
        initComponents();
        initConnectFields(); 
        nullModel = dataTable.getModel();  //Save null TableModel
        tableSelectionModel = dataTable.getSelectionModel(); 
        tableSelectionModel.addListSelectionListener(
                            new TableSelectionHandler());
        //dataTable.setDefaultRenderer(JLabel.class, new auditColumnRenderer() );
        dataTable.setDefaultRenderer(String.class, new auditColumnRenderer() );
        //dataTable.getColumnModel().getColumn(1).setCellRenderer(new auditColumnRenderer());
        
        for ( int i=0; i<100; i++) {
            selectedIndexes[i] = 0; 
            selectedViewIndexes[i] = 0; 
        }
    }
    
    class TableSelectionHandler implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) {
            ListSelectionModel lsm = (ListSelectionModel)e.getSource();  
            
            //System.out.print( "Table Selection Trigger"); 
            //if ( loadInProgress ) System.out.print( " LIP "); else System.out.print(" No LIP "); 
            //if ( lsm.isSelectionEmpty() ) System.out.print( " Empty "); else System.out.print(" "+dataTable.getSelectedRowCount()+" Sel "); 
            //System.out.print( "\n"); 
           if ( !e.getValueIsAdjusting()) {
            if ( !loadInProgress ) {
             if (lsm.isSelectionEmpty() ) {
                alterButton.setEnabled(false);
             } 
             else { 
                alterButton.setEnabled(true); 
                //System.out.println( "Selection Not Empty");                

                selectedViewIndexes = dataTable.getSelectedRows();   //Gets view rows    
                numSelected = dataTable.getSelectedRowCount(); 
                System.out.println( "Num Selected: "+numSelected ); 
                System.out.println("Index length: "+selectedIndexes.length+"  View Length: "+selectedViewIndexes.length ); 
                for ( int i=0; i<numSelected; i++ ) {
                    System.out.println("i: "+i+"  viewIndex: "+selectedViewIndexes[i]+"  index: "+dataTable.convertRowIndexToModel( selectedViewIndexes[i] ));
                    selectedIndexes[i] = dataTable.convertRowIndexToModel( selectedViewIndexes[i] ); 
                }
                if ( alterFrame.isVisible() ) {  //If Alter Screen is visible......                    
                    alterInit();   //...re-set the Alter Screen to match the selection. 
                    alterFrame.toFront(); 
                }
             }
            }
           } 
        }
    } 
    
    
    public class auditColumnRenderer extends JLabel implements TableCellRenderer {
        
        public auditColumnRenderer() {
            setOpaque(true); 
        }    
        
        public Component getTableCellRendererComponent( JTable table, Object value, boolean isSelected,
                                      boolean hasFocus, int row, int column) {
            String sqlMatchTbl = "SELECT triggerman.compareAuditTbl(?, ?, ?, ?) FROM dual"; 
            PreparedStatement ps; 
            ResultSet rs;
            boolean match; 
            //System.out.println("Audit Render"); 
           try { 
               
            //setText(dataTable.getValueAt(row, column).toString()); 
            setText(value.toString()); 
            setFont(new java.awt.Font("Tahoma", 1, 14)); 

            if ( isSelected)
                setBackground(Color.lightGray); 
            else
                setBackground(Color.white); 
            
            if ( column == 1 ) {
                //System.out.println( "Col 1"); 
             if ( !dataTable.getValueAt(row, 1).toString().isEmpty()) {
                 
                ps = infoConn.prepareStatement(sqlMatchTbl);
                ps.setString(1, dataTable.getValueAt(row, 0).toString());
                ps.setString(2, schemaName);
                ps.setString(3, dataTable.getValueAt(row, 1).toString());
                ps.setString(4, auditSchemaName);
                rs = ps.executeQuery(); rs.next(); 
                match = rs.getBoolean(1); 
                
                if ( !match )
                    setForeground(Color.red);
                else 
                    setForeground(Color.black); 
             } 
             else   //No Audit Table
                setForeground(Color.black);                
            }
           else  //Not Audit Table Column
                setForeground(Color.black); 
                //return table.getCellRenderer(row, column).getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column); 
            
            return this; 
           } 
           catch (Exception e) {
            e.printStackTrace( System.err );
            /*  JOptionPane.showMessageDialog(this,
                "Audit Render: " + e.getMessage(),
                null, JOptionPane.ERROR_MESSAGE);
            */
            return null; 
           } 
           
        }     
    }

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        connectFrame = new javax.swing.JFrame();
        usernameTextField = new javax.swing.JTextField();
        connectOKButton = new javax.swing.JButton();
        connectCancelButton = new javax.swing.JButton();
        usernameLabel = new javax.swing.JLabel();
        passwordLabel = new javax.swing.JLabel();
        passwordPasswordField = new javax.swing.JPasswordField();
        connectTextField = new javax.swing.JTextField();
        connectComboBox = new javax.swing.JComboBox();
        connectConnectButton = new javax.swing.JButton();
        connectURLLabel = new javax.swing.JLabel();
        connectURL = new javax.swing.JTextField();
        alterFrame = new javax.swing.JFrame();
        alterTableNameLabel = new javax.swing.JLabel();
        alterAuditTableNameLabel = new javax.swing.JLabel();
        alterBDTriggerNameLabel = new javax.swing.JLabel();
        alterBITriggerNameLabel = new javax.swing.JLabel();
        alterBUTriggerNameLabel = new javax.swing.JLabel();
        alterADTriggerNameLabel = new javax.swing.JLabel();
        alterAITriggerNameLabel = new javax.swing.JLabel();
        alterAUTriggerNameLabel = new javax.swing.JLabel();
        alterTableName = new javax.swing.JTextField();
        alterAuditTableName = new javax.swing.JTextField();
        alterBDTriggerName = new javax.swing.JTextField();
        alterBITriggerName = new javax.swing.JTextField();
        alterBUTriggerName = new javax.swing.JTextField();
        alterADTriggerName = new javax.swing.JTextField();
        alterAITriggerName = new javax.swing.JTextField();
        alterAUTriggerName = new javax.swing.JTextField();
        alterBDCheckBox = new javax.swing.JCheckBox();
        alterBICheckBox = new javax.swing.JCheckBox();
        alterBUCheckBox = new javax.swing.JCheckBox();
        alterADCheckBox = new javax.swing.JCheckBox();
        alterAICheckBox = new javax.swing.JCheckBox();
        alterAUCheckBox = new javax.swing.JCheckBox();
        alterDropTriggersButton = new javax.swing.JButton();
        alterCreateTriggersButton = new javax.swing.JButton();
        alterDropAuditTableButton = new javax.swing.JButton();
        alterCreateAuditTableButton = new javax.swing.JButton();
        alterExitButton = new javax.swing.JButton();
        alterAuditCheckBox = new javax.swing.JCheckBox();
        optionsFrame = new javax.swing.JFrame();
        optionsHeaderLabel = new javax.swing.JLabel();
        optionsSchemaNameLabel = new javax.swing.JLabel();
        optionsSchemaName = new javax.swing.JTextField();
        optionsAuditPrefixLabel = new javax.swing.JLabel();
        optionsAuditPrefix = new javax.swing.JTextField();
        optionsAuditSuffixLabel = new javax.swing.JLabel();
        optionsAuditSuffix = new javax.swing.JTextField();
        jLabel2 = new javax.swing.JLabel();
        optionsTriggerNameFormatString = new javax.swing.JTextField();
        optionsBeforeLabel = new javax.swing.JLabel();
        optionsBeforeString = new javax.swing.JTextField();
        optionsAfterLabel = new javax.swing.JLabel();
        optionsAfterString = new javax.swing.JTextField();
        optionsDeleteLabel = new javax.swing.JLabel();
        optionsDeleteString = new javax.swing.JTextField();
        optionsInsertLabel = new javax.swing.JLabel();
        optionsInsertString = new javax.swing.JTextField();
        optionsUpdateLabel = new javax.swing.JLabel();
        optionsUpdateString = new javax.swing.JTextField();
        optionsAuditSchemaLabel = new javax.swing.JLabel();
        optionsAuditSchemaComboBox = new javax.swing.JComboBox();
        optionsSaveButton = new javax.swing.JButton();
        optionsExitButton = new javax.swing.JButton();
        tmpAuditTablePrefixLabel = new javax.swing.JLabel();
        optionsTmpAuditPrefix = new javax.swing.JTextField();
        optionsBDCheckBox = new javax.swing.JCheckBox();
        optionsBICheckBox = new javax.swing.JCheckBox();
        optionsBUCheckBox = new javax.swing.JCheckBox();
        optionsADCheckBox = new javax.swing.JCheckBox();
        optionsAICheckBox = new javax.swing.JCheckBox();
        optionsAUCheckBox = new javax.swing.JCheckBox();
        optionsSaveRestoreCheckBox = new javax.swing.JCheckBox();
        optionsMultiLabel = new javax.swing.JLabel();
        optionsRebuildAuditCheckBox = new javax.swing.JCheckBox();
        optionsDefaultTriggersLabel = new javax.swing.JLabel();
        aboutFrame = new javax.swing.JFrame();
        jScrollPane2 = new javax.swing.JScrollPane();
        aboutTextPane = new javax.swing.JTextPane();
        impiricalCheckBox = new javax.swing.JCheckBox();
        OSICheckBox = new javax.swing.JCheckBox();
        jScrollPane1 = new javax.swing.JScrollPane();
        dataTable = new javax.swing.JTable();
        connectButton = new javax.swing.JButton();
        statusLabel = new javax.swing.JLabel();
        databaseLabel = new javax.swing.JLabel();
        statusTextField = new javax.swing.JTextField();
        schemaTextField = new javax.swing.JTextField();
        disconnectButton = new javax.swing.JButton();
        alterButton = new javax.swing.JButton();
        auditSchemaLabel = new javax.swing.JLabel();
        auditSchemaTextField = new javax.swing.JTextField();
        optionsButton = new javax.swing.JButton();
        aboutButton = new javax.swing.JButton();

        connectFrame.setTitle("Connect");
        connectFrame.setAlwaysOnTop(true);
        connectFrame.setMinimumSize(new java.awt.Dimension(383, 325));
        connectFrame.setResizable(false);

        usernameTextField.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        connectOKButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        connectOKButton.setText("OK");
        connectOKButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                connectOKButtonActionPerformed(evt);
            }
        });

        connectCancelButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        connectCancelButton.setText("Cancel");
        connectCancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                connectCancelButtonActionPerformed(evt);
            }
        });

        usernameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        usernameLabel.setText("UserName:");

        passwordLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        passwordLabel.setText("Password:");

        passwordPasswordField.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        connectTextField.setEditable(false);
        connectTextField.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        connectTextField.setFocusable(false);

        connectComboBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        connectConnectButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        connectConnectButton.setText("Connect");
        connectConnectButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                connectConnectButtonActionPerformed(evt);
            }
        });

        connectURLLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        connectURLLabel.setText("URL:");

        connectURL.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        javax.swing.GroupLayout connectFrameLayout = new javax.swing.GroupLayout(connectFrame.getContentPane());
        connectFrame.getContentPane().setLayout(connectFrameLayout);
        connectFrameLayout.setHorizontalGroup(
            connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(connectFrameLayout.createSequentialGroup()
                .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, connectFrameLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(connectTextField))
                    .addGroup(connectFrameLayout.createSequentialGroup()
                        .addGap(36, 36, 36)
                        .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(connectFrameLayout.createSequentialGroup()
                                .addComponent(connectOKButton)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(connectCancelButton))
                            .addComponent(connectConnectButton, javax.swing.GroupLayout.PREFERRED_SIZE, 98, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(connectFrameLayout.createSequentialGroup()
                                .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addComponent(connectURLLabel)
                                    .addComponent(usernameLabel)
                                    .addComponent(passwordLabel))
                                .addGap(18, 18, 18)
                                .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                                        .addComponent(usernameTextField, javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(connectURL, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.PREFERRED_SIZE, 223, javax.swing.GroupLayout.PREFERRED_SIZE))
                                    .addComponent(passwordPasswordField, javax.swing.GroupLayout.PREFERRED_SIZE, 223, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addComponent(connectComboBox, 0, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                        .addGap(20, 20, 20)))
                .addContainerGap())
        );
        connectFrameLayout.setVerticalGroup(
            connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(connectFrameLayout.createSequentialGroup()
                .addGap(9, 9, 9)
                .addComponent(connectTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(connectURL, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(connectURLLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(usernameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(usernameLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(passwordPasswordField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(passwordLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 18, Short.MAX_VALUE)
                .addComponent(connectConnectButton)
                .addGap(18, 18, 18)
                .addComponent(connectComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addGroup(connectFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(connectOKButton)
                    .addComponent(connectCancelButton))
                .addContainerGap())
        );

        alterFrame.setTitle("Alter");
        alterFrame.setMinimumSize(new java.awt.Dimension(627, 577));
        alterFrame.setName("");
        alterFrame.setResizable(false);

        alterTableNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterTableNameLabel.setText("Table Name:");
        alterTableNameLabel.setFocusable(false);

        alterAuditTableNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterAuditTableNameLabel.setText("Audit Table Name: ");
        alterAuditTableNameLabel.setFocusable(false);

        alterBDTriggerNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBDTriggerNameLabel.setText("BD Trigger Name:");

        alterBITriggerNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBITriggerNameLabel.setText("BI Trigger Name:");

        alterBUTriggerNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBUTriggerNameLabel.setText("BU Trigger Name:");

        alterADTriggerNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterADTriggerNameLabel.setText("AD Trigger Name:");

        alterAITriggerNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterAITriggerNameLabel.setText("AI Trigger Name:");

        alterAUTriggerNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterAUTriggerNameLabel.setText("AU Trigger Name:");

        alterTableName.setEditable(false);
        alterTableName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterTableName.setFocusable(false);

        alterAuditTableName.setEditable(false);
        alterAuditTableName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterAuditTableName.setFocusable(false);

        alterBDTriggerName.setEditable(false);
        alterBDTriggerName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBDTriggerName.setFocusable(false);
        alterBDTriggerName.setName("alterBDTriggerName");

        alterBITriggerName.setEditable(false);
        alterBITriggerName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBITriggerName.setFocusable(false);
        alterBITriggerName.setName("alterBITriggerName");

        alterBUTriggerName.setEditable(false);
        alterBUTriggerName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBUTriggerName.setFocusable(false);
        alterBUTriggerName.setName("alterBUTriggerName");

        alterADTriggerName.setEditable(false);
        alterADTriggerName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterADTriggerName.setFocusable(false);
        alterADTriggerName.setName("alterADTriggerName");

        alterAITriggerName.setEditable(false);
        alterAITriggerName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterAITriggerName.setFocusable(false);
        alterAITriggerName.setName("alterAITriggerName");

        alterAUTriggerName.setEditable(false);
        alterAUTriggerName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterAUTriggerName.setFocusable(false);
        alterAUTriggerName.setName("alterAUTriggerName");

        alterBDCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBDCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterBDCheckBoxActionPerformed(evt);
            }
        });

        alterBICheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBICheckBox.setName("");
        alterBICheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterBICheckBoxActionPerformed(evt);
            }
        });

        alterBUCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterBUCheckBox.setName("");
        alterBUCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterBUCheckBoxActionPerformed(evt);
            }
        });

        alterADCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterADCheckBox.setName("");
        alterADCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterADCheckBoxActionPerformed(evt);
            }
        });

        alterAICheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterAICheckBox.setName("");
        alterAICheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterAICheckBoxActionPerformed(evt);
            }
        });

        alterAUCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterAUCheckBox.setName("");
        alterAUCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterAUCheckBoxActionPerformed(evt);
            }
        });

        alterDropTriggersButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterDropTriggersButton.setText("Drop Triggers");
        alterDropTriggersButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterDropTriggersButtonActionPerformed(evt);
            }
        });

        alterCreateTriggersButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterCreateTriggersButton.setText("Create Triggers");
        alterCreateTriggersButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterCreateTriggersButtonActionPerformed(evt);
            }
        });

        alterDropAuditTableButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterDropAuditTableButton.setText("Drop Audit Table");
        alterDropAuditTableButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterDropAuditTableButtonActionPerformed(evt);
            }
        });

        alterCreateAuditTableButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterCreateAuditTableButton.setText("Create Audit Table");
        alterCreateAuditTableButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterCreateAuditTableButtonActionPerformed(evt);
            }
        });

        alterExitButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterExitButton.setText("Exit");
        alterExitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterExitButtonActionPerformed(evt);
            }
        });

        alterAuditCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterAuditCheckBoxActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout alterFrameLayout = new javax.swing.GroupLayout(alterFrame.getContentPane());
        alterFrame.getContentPane().setLayout(alterFrameLayout);
        alterFrameLayout.setHorizontalGroup(
            alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(alterFrameLayout.createSequentialGroup()
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addGroup(alterFrameLayout.createSequentialGroup()
                        .addContainerGap()
                        .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(alterBUCheckBox)
                            .addComponent(alterBICheckBox)
                            .addComponent(alterBDCheckBox)
                            .addComponent(alterAICheckBox)
                            .addComponent(alterADCheckBox)
                            .addComponent(alterAUCheckBox)
                            .addComponent(alterAuditCheckBox))
                        .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(alterFrameLayout.createSequentialGroup()
                                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(alterFrameLayout.createSequentialGroup()
                                            .addGap(63, 63, 63)
                                            .addComponent(alterTableNameLabel)
                                            .addGap(3, 3, 3))
                                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, alterFrameLayout.createSequentialGroup()
                                            .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                            .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(alterBDTriggerNameLabel, javax.swing.GroupLayout.Alignment.TRAILING)
                                                .addComponent(alterBITriggerNameLabel, javax.swing.GroupLayout.Alignment.TRAILING)
                                                .addComponent(alterBUTriggerNameLabel, javax.swing.GroupLayout.Alignment.TRAILING))))
                                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, alterFrameLayout.createSequentialGroup()
                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                        .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                .addComponent(alterADTriggerNameLabel)
                                                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                                    .addComponent(alterAUTriggerNameLabel)
                                                    .addComponent(alterAITriggerNameLabel)))
                                            .addComponent(alterAuditTableNameLabel, javax.swing.GroupLayout.Alignment.TRAILING))))
                                .addGap(18, 18, 18)
                                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                        .addComponent(alterAuditTableName, javax.swing.GroupLayout.DEFAULT_SIZE, 332, Short.MAX_VALUE)
                                        .addComponent(alterBDTriggerName)
                                        .addComponent(alterBITriggerName)
                                        .addComponent(alterBUTriggerName)
                                        .addComponent(alterADTriggerName))
                                    .addComponent(alterTableName, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 332, javax.swing.GroupLayout.PREFERRED_SIZE)
                                    .addComponent(alterAITriggerName, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 332, javax.swing.GroupLayout.PREFERRED_SIZE)))
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, alterFrameLayout.createSequentialGroup()
                                .addGap(170, 170, 170)
                                .addComponent(alterAUTriggerName, javax.swing.GroupLayout.PREFERRED_SIZE, 332, javax.swing.GroupLayout.PREFERRED_SIZE))))
                    .addGroup(alterFrameLayout.createSequentialGroup()
                        .addGap(115, 115, 115)
                        .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(alterCreateAuditTableButton)
                            .addComponent(alterDropAuditTableButton, javax.swing.GroupLayout.PREFERRED_SIZE, 163, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                        .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(alterDropTriggersButton, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 141, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(alterCreateTriggersButton, javax.swing.GroupLayout.Alignment.TRAILING))))
                .addContainerGap(98, Short.MAX_VALUE))
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, alterFrameLayout.createSequentialGroup()
                .addGap(0, 0, Short.MAX_VALUE)
                .addComponent(alterExitButton)
                .addGap(269, 269, 269))
        );
        alterFrameLayout.setVerticalGroup(
            alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(alterFrameLayout.createSequentialGroup()
                .addGap(25, 25, 25)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(alterTableNameLabel, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(alterTableName, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(alterAuditTableName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(alterAuditTableNameLabel))
                    .addComponent(alterAuditCheckBox))
                .addGap(22, 22, 22)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(alterBDTriggerName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(alterBDTriggerNameLabel))
                    .addComponent(alterBDCheckBox))
                .addGap(18, 18, 18)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(alterBITriggerName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(alterBITriggerNameLabel))
                    .addComponent(alterBICheckBox))
                .addGap(18, 18, 18)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(alterBUTriggerName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(alterBUTriggerNameLabel))
                    .addComponent(alterBUCheckBox))
                .addGap(18, 18, 18)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(alterADTriggerName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(alterADTriggerNameLabel))
                    .addComponent(alterADCheckBox))
                .addGap(18, 18, 18)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(alterAITriggerName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(alterAITriggerNameLabel))
                    .addComponent(alterAICheckBox))
                .addGap(18, 18, 18)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(alterAUCheckBox)
                    .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(alterAUTriggerName, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(alterAUTriggerNameLabel)))
                .addGap(50, 50, 50)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(alterDropAuditTableButton, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(alterDropTriggersButton))
                .addGap(18, 18, 18)
                .addGroup(alterFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(alterCreateAuditTableButton)
                    .addComponent(alterCreateTriggersButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 50, Short.MAX_VALUE)
                .addComponent(alterExitButton)
                .addGap(39, 39, 39))
        );

        alterAuditTableName.getAccessibleContext().setAccessibleDescription("");

        optionsFrame.setTitle("Options");
        optionsFrame.setMinimumSize(new java.awt.Dimension(613, 820));
        optionsFrame.setResizable(false);

        optionsHeaderLabel.setFont(new java.awt.Font("Tahoma", 1, 18)); // NOI18N
        optionsHeaderLabel.setText("TriggerMan Options");

        optionsSchemaNameLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsSchemaNameLabel.setText("Schema Name: ");

        optionsSchemaName.setEditable(false);
        optionsSchemaName.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsSchemaName.setFocusable(false);

        optionsAuditPrefixLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsAuditPrefixLabel.setText("Audit Table Prefix:");

        optionsAuditPrefix.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsAuditSuffixLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsAuditSuffixLabel.setText("Audit Table Suffix:");

        optionsAuditSuffix.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        jLabel2.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        jLabel2.setText("Trigger Name Format:");

        optionsTriggerNameFormatString.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsBeforeLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsBeforeLabel.setText("BEFORE String:");

        optionsBeforeString.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsAfterLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsAfterLabel.setText("AFTER String:");

        optionsAfterString.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsDeleteLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsDeleteLabel.setText("DELETE String:");

        optionsDeleteString.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsInsertLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsInsertLabel.setText("INSERT String:");

        optionsInsertString.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsUpdateLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsUpdateLabel.setText("UPDATE String:");

        optionsUpdateString.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsAuditSchemaLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsAuditSchemaLabel.setText("Audit Table Schema:");

        optionsAuditSchemaComboBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsSaveButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsSaveButton.setText("Save");
        optionsSaveButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                optionsSaveButtonActionPerformed(evt);
            }
        });

        optionsExitButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsExitButton.setText("Exit");
        optionsExitButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                optionsExitButtonActionPerformed(evt);
            }
        });

        tmpAuditTablePrefixLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        tmpAuditTablePrefixLabel.setText("Temp. Audit Table Prefix: ");

        optionsTmpAuditPrefix.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N

        optionsBDCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsBDCheckBox.setText("BD");

        optionsBICheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsBICheckBox.setText("BI");

        optionsBUCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsBUCheckBox.setText("BU");

        optionsADCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsADCheckBox.setText("AD");

        optionsAICheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsAICheckBox.setText("AI");

        optionsAUCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsAUCheckBox.setText("AU");

        optionsSaveRestoreCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsSaveRestoreCheckBox.setText("Save/Restore Audit Data");

        optionsMultiLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsMultiLabel.setText("Multi Options:");

        optionsRebuildAuditCheckBox.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsRebuildAuditCheckBox.setText("Rebuild Audit Tables");
        optionsRebuildAuditCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                optionsRebuildAuditCheckBoxActionPerformed(evt);
            }
        });

        optionsDefaultTriggersLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsDefaultTriggersLabel.setText("Default Triggers:");

        javax.swing.GroupLayout optionsFrameLayout = new javax.swing.GroupLayout(optionsFrame.getContentPane());
        optionsFrame.getContentPane().setLayout(optionsFrameLayout);
        optionsFrameLayout.setHorizontalGroup(
            optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(optionsFrameLayout.createSequentialGroup()
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(optionsFrameLayout.createSequentialGroup()
                        .addGap(122, 122, 122)
                        .addComponent(optionsSchemaNameLabel)
                        .addGap(18, 18, 18)
                        .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(optionsAuditPrefix, javax.swing.GroupLayout.DEFAULT_SIZE, 218, Short.MAX_VALUE)
                            .addComponent(optionsAuditSuffix, javax.swing.GroupLayout.DEFAULT_SIZE, 218, Short.MAX_VALUE)
                            .addComponent(optionsSchemaName)))
                    .addGroup(optionsFrameLayout.createSequentialGroup()
                        .addGap(209, 209, 209)
                        .addComponent(optionsHeaderLabel))
                    .addGroup(optionsFrameLayout.createSequentialGroup()
                        .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(optionsFrameLayout.createSequentialGroup()
                                .addGap(45, 45, 45)
                                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                    .addComponent(jLabel2)
                                    .addComponent(optionsAfterLabel)
                                    .addComponent(optionsDeleteLabel)
                                    .addComponent(optionsInsertLabel)
                                    .addComponent(optionsUpdateLabel)
                                    .addComponent(tmpAuditTablePrefixLabel)
                                    .addComponent(optionsAuditSchemaLabel)
                                    .addComponent(optionsAuditSuffixLabel)
                                    .addComponent(optionsAuditPrefixLabel)
                                    .addComponent(optionsMultiLabel))
                                .addGap(25, 25, 25))
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, optionsFrameLayout.createSequentialGroup()
                                .addContainerGap()
                                .addComponent(optionsBeforeLabel)
                                .addGap(24, 24, 24)))
                        .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(optionsRebuildAuditCheckBox)
                            .addGroup(optionsFrameLayout.createSequentialGroup()
                                .addComponent(optionsADCheckBox)
                                .addGap(18, 18, 18)
                                .addComponent(optionsAICheckBox)
                                .addGap(18, 18, 18)
                                .addComponent(optionsAUCheckBox))
                            .addGroup(optionsFrameLayout.createSequentialGroup()
                                .addComponent(optionsBDCheckBox)
                                .addGap(18, 18, 18)
                                .addComponent(optionsBICheckBox)
                                .addGap(18, 18, 18)
                                .addComponent(optionsBUCheckBox))
                            .addComponent(optionsSaveRestoreCheckBox)
                            .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                .addGroup(optionsFrameLayout.createSequentialGroup()
                                    .addComponent(optionsSaveButton)
                                    .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                    .addComponent(optionsExitButton))
                                .addComponent(optionsTmpAuditPrefix)
                                .addComponent(optionsTriggerNameFormatString)
                                .addComponent(optionsBeforeString, javax.swing.GroupLayout.Alignment.TRAILING)
                                .addComponent(optionsAfterString, javax.swing.GroupLayout.Alignment.TRAILING)
                                .addComponent(optionsDeleteString, javax.swing.GroupLayout.Alignment.TRAILING)
                                .addComponent(optionsInsertString, javax.swing.GroupLayout.Alignment.TRAILING)
                                .addComponent(optionsUpdateString, javax.swing.GroupLayout.Alignment.TRAILING)
                                .addComponent(optionsAuditSchemaComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 218, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addComponent(optionsDefaultTriggersLabel))))
                .addContainerGap(148, Short.MAX_VALUE))
        );
        optionsFrameLayout.setVerticalGroup(
            optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(optionsFrameLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(optionsHeaderLabel)
                .addGap(34, 34, 34)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(optionsSchemaNameLabel, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(optionsSchemaName, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(21, 21, 21)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(optionsAuditPrefix, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(optionsAuditPrefixLabel, javax.swing.GroupLayout.Alignment.TRAILING))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(optionsAuditSuffix, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(optionsAuditSuffixLabel, javax.swing.GroupLayout.Alignment.TRAILING))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(optionsAuditSchemaComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(optionsAuditSchemaLabel))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(tmpAuditTablePrefixLabel, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(optionsTmpAuditPrefix, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(optionsTriggerNameFormatString, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2, javax.swing.GroupLayout.Alignment.TRAILING))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(optionsBeforeLabel, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(optionsBeforeString, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(optionsAfterLabel)
                    .addComponent(optionsAfterString, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(optionsDeleteLabel)
                    .addComponent(optionsDeleteString, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(optionsInsertLabel)
                    .addComponent(optionsInsertString, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(optionsUpdateLabel)
                    .addComponent(optionsUpdateString, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(optionsMultiLabel)
                    .addComponent(optionsDefaultTriggersLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(optionsBDCheckBox)
                    .addComponent(optionsBICheckBox)
                    .addComponent(optionsBUCheckBox))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(optionsADCheckBox)
                    .addComponent(optionsAICheckBox)
                    .addComponent(optionsAUCheckBox))
                .addGap(18, 18, 18)
                .addComponent(optionsRebuildAuditCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(optionsSaveRestoreCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 54, Short.MAX_VALUE)
                .addGroup(optionsFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(optionsSaveButton)
                    .addComponent(optionsExitButton))
                .addGap(23, 23, 23))
        );

        aboutFrame.setTitle("About");
        aboutFrame.setBackground(new java.awt.Color(255, 255, 255));
        aboutFrame.setForeground(java.awt.Color.white);
        aboutFrame.setMinimumSize(new java.awt.Dimension(556, 422));
        aboutFrame.setResizable(false);
        aboutFrame.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                aboutFrameMouseClicked(evt);
            }
        });

        aboutTextPane.setEditable(false);
        aboutTextPane.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        aboutTextPane.setText("TriggerMan - An Audit Table and Trigger Management Tool for MySQL. \nVersion 1.0 - April 01, 2013\n\n© 2013 Impirical Software Engineering   www.impiricalsoftware.com\nDavid J. Hutira   djhutira@impiricalsoftware.com\n\nTriggerMan was built using NetBeans 7.1.2, MySQL 5.2 and 7-zip 9.2.  \n\nUses the O'Reilly ResultSetTableModel/Factory\n© 2009 O'Reilly Media, Inc. All Rights Reserved. \n\n    This program is free software: you can redistribute it and/or modify\n    it under the terms of the GNU General Public License as published by\n    the Free Software Foundation, Version 3 of the License. \n\n    This program is distributed in the hope that it will be useful,\n    but WITHOUT ANY WARRANTY; without even the implied warranty \n    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  \n    See the GNU General Public License for more details.\n\n    You should have received a copy of the GNU General Public License\n    along with this program.  If not, see www.gnu.org/licenses/.");
        aboutTextPane.setFocusable(false);
        jScrollPane2.setViewportView(aboutTextPane);

        impiricalCheckBox.setFocusable(false);
        impiricalCheckBox.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/Impirical SE.jpg"))); // NOI18N

        OSICheckBox.setIcon(new javax.swing.ImageIcon(getClass().getResource("/images/osi_standard_logo_qtr.png"))); // NOI18N

        javax.swing.GroupLayout aboutFrameLayout = new javax.swing.GroupLayout(aboutFrame.getContentPane());
        aboutFrame.getContentPane().setLayout(aboutFrameLayout);
        aboutFrameLayout.setHorizontalGroup(
            aboutFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(aboutFrameLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(aboutFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(aboutFrameLayout.createSequentialGroup()
                        .addComponent(impiricalCheckBox)
                        .addGap(18, 18, 18)
                        .addComponent(OSICheckBox, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE))
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 536, Short.MAX_VALUE))
                .addContainerGap())
        );
        aboutFrameLayout.setVerticalGroup(
            aboutFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(aboutFrameLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane2, javax.swing.GroupLayout.PREFERRED_SIZE, 247, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(28, 28, 28)
                .addGroup(aboutFrameLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(impiricalCheckBox, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(OSICheckBox, javax.swing.GroupLayout.PREFERRED_SIZE, 0, Short.MAX_VALUE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("TriggerMan");

        dataTable.setAutoCreateRowSorter(true);
        dataTable.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        dataTable.setGridColor(new java.awt.Color(0, 0, 0));
        dataTable.setSelectionMode(javax.swing.ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        dataTable.getTableHeader().setReorderingAllowed(false);
        jScrollPane1.setViewportView(dataTable);

        connectButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        connectButton.setText("Connect");
        connectButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                connectButtonActionPerformed(evt);
            }
        });

        statusLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        statusLabel.setText("Status: ");

        databaseLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        databaseLabel.setText("Table Schema:");

        statusTextField.setEditable(false);
        statusTextField.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        statusTextField.setFocusable(false);

        schemaTextField.setEditable(false);
        schemaTextField.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        schemaTextField.setFocusable(false);

        disconnectButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        disconnectButton.setText("Disconnect");
        disconnectButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                disconnectButtonActionPerformed(evt);
            }
        });

        alterButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        alterButton.setText("Alter");
        alterButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                alterButtonActionPerformed(evt);
            }
        });

        auditSchemaLabel.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        auditSchemaLabel.setText("Audit Schema: ");

        auditSchemaTextField.setEditable(false);
        auditSchemaTextField.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        auditSchemaTextField.setFocusable(false);

        optionsButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        optionsButton.setText("Options");
        optionsButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                optionsButtonActionPerformed(evt);
            }
        });

        aboutButton.setFont(new java.awt.Font("Tahoma", 1, 14)); // NOI18N
        aboutButton.setText("About");
        aboutButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutButtonActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 1088, Short.MAX_VALUE)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                .addComponent(auditSchemaLabel)
                                .addComponent(databaseLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 104, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addComponent(statusLabel, javax.swing.GroupLayout.Alignment.TRAILING))
                            .addComponent(connectButton))
                        .addGap(7, 7, 7)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addGroup(layout.createSequentialGroup()
                                .addGap(10, 10, 10)
                                .addComponent(disconnectButton)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 30, Short.MAX_VALUE)
                                .addComponent(alterButton))
                            .addComponent(statusTextField, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 216, Short.MAX_VALUE)
                            .addComponent(schemaTextField)
                            .addComponent(auditSchemaTextField))
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGap(33, 33, 33)
                                .addComponent(optionsButton)
                                .addGap(0, 0, Short.MAX_VALUE))
                            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(aboutButton)))))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusLabel)
                    .addComponent(statusTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(aboutButton))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(schemaTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(databaseLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(auditSchemaTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(auditSchemaLabel))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(connectButton)
                    .addComponent(disconnectButton)
                    .addComponent(alterButton)
                    .addComponent(optionsButton))
                .addGap(9, 9, 9)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 374, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void connectButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_connectButtonActionPerformed

               
        connectTextField.setText("Enter Username/Password, then click Connect");
        connectURL.setText( urlPrefix ); 
        usernameTextField.setText(null); 
        passwordPasswordField.setText(null); 
        acct = null; pwd = null; 
        
        //usernameTextField.requestFocusInWindow(); 
        connectFrame.setVisible(true);
        usernameTextField.requestFocusInWindow();         
    }//GEN-LAST:event_connectButtonActionPerformed

    private void connectCancelButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_connectCancelButtonActionPerformed

        disconnectTman(); 
        connectFrame.setVisible(false);
                
    }//GEN-LAST:event_connectCancelButtonActionPerformed

    private void connectOKButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_connectOKButtonActionPerformed

        //Now try to connect to the database the user has selected
        try{
           
        Class.forName("com.mysql.jdbc.Driver");
        char[] pwd = passwordPasswordField.getPassword(); 
        urlPrefix = connectURL.getText(); //Allow user to change MySQL URL. 
        //System.err.println( "User: " + usernameTextField.getText() + "  Password: " + String.valueOf(input) ); 
        conn = DriverManager.getConnection( urlPrefix + connectComboBox.getSelectedItem().toString(), usernameTextField.getText(), String.valueOf(pwd) );  
        
        // If we make it here, we assume the connection was successful.       
        
        connectFrame.setVisible(false);    
        
        schemaName = connectComboBox.getSelectedItem().toString(); 
        
        statusTextField.setText("Connected"); 
        schemaTextField.setText(schemaName); 
        
        connectButton.setEnabled(false);
        disconnectButton.setEnabled(true); 
        optionsButton.setEnabled(true); 
        
        loadTManInfo();
        loadTableInfo(); 
       }        
       catch (Exception e) {        
           // Connection attempt failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            connectComboBox.getSelectedItem().toString() + ": " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
       }       
    }//GEN-LAST:event_connectOKButtonActionPerformed

    private void connectConnectButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_connectConnectButtonActionPerformed
        
       //Attempt to make JDBC connection to info schema to show list of schemas. 
       boolean installed;  
       try{
           
        Class.forName("com.mysql.jdbc.Driver");
        char[] input = passwordPasswordField.getPassword(); 
        //System.err.println( "User: " + usernameTextField.getText() + "  Password: " + String.valueOf(input) ); 
        infoConn = DriverManager.getConnection( infoSchemaURL, usernameTextField.getText(), String.valueOf(input) );  
        
        // If we make it here, we assume the connection was successful. 
        
        acct = usernameTextField.getText(); 
        pwd = String.valueOf(input); 
        
        // Perform TriggerMan checks and possibly Initialization
        
        installed = checkTManInit();
        
        if ( installed ) {
        //Bring up a list of schemas for the user to choose from.     
            connectTextField.setText("Select DB, then click OK");  
            loadSchemaNames( connectComboBox ); 
        }
        else {  // TMan not installed, exit gracefully...
            disconnectTman(); 
            connectFrame.setVisible(false);
        }
       }        
       catch (Exception e) {           
           // Connection attempt failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "InfoSchema: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
       }
    }//GEN-LAST:event_connectConnectButtonActionPerformed

    private void loadSchemaNames( JComboBox box) {
        //SQLNeeded
        String sqlSchemaAll = "SELECT schema_name FROM schemata WHERE schema_name NOT IN ( 'information_schema', 'performance_schema', 'mysql', 'triggerman' )";
        PreparedStatement schemaStmt;
        ResultSet schemaRS;
   
       try {
        
        schemaStmt = infoConn.prepareStatement(sqlSchemaAll);
        schemaRS = schemaStmt.executeQuery();

        box.removeAllItems(); 
        while (schemaRS.next()) {
            box.addItem(schemaRS.getString("schema_name"));
        }
       }  
             
       catch (Exception e) {
           
           // Connection attempt failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "LoadSChemaNames: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);

       }
    }
    
    
    private void disconnectButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_disconnectButtonActionPerformed

        //Disconnect drops both the info schema and DB links,        
        disconnectTman(); 
    }//GEN-LAST:event_disconnectButtonActionPerformed

    private void alterButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterButtonActionPerformed
 
        //selectedIndexes = dataTable.getSelectedRows();       
        //numSelected = dataTable.getSelectedRowCount(); 
        alterInit(); 
        
        alterFrame.setVisible(true);       
        
    }//GEN-LAST:event_alterButtonActionPerformed

    private void alterInit() {
        int rowSelected; 
        
        // First check the user's selection...
        //selectedIndexes = dataTable.getSelectedRows();       
        //numSelected = dataTable.getSelectedRowCount(); 
        //System.out.println( "Alter Init"); 
        initAlterFields(); 
        
        if ( numSelected == 0 ) return; 
        
        if ( numSelected == 1 ) {
 
            rowSelected = dataTable.convertRowIndexToView(selectedIndexes[0]);
            //rowSelected = selectedIndexes[0];
            //System.out.println( "Table Index: " + rowSelected + "    View Index:  " + selectedIndexes[0] );
                    
            // Now load single choice into alterFrame. 
            loadAlterFieldsSingle( rowSelected ); 
                   
        }
        else {
        //MultiSelection processing
            
            //Load multiple object data into Alter Screen fields.                 
            loadAlterFieldsMulti( selectedIndexes ); 
            
        }
    }
    
    private void loadAlterFieldsSingle( int rowSelected ) {

            selectedTableName = dataTable.getValueAt( rowSelected, 0).toString(); // rowSelected already translated to View
            alterTableName.setText( selectedTableName ); 
            
            selectedAuditTableName = dataTable.getValueAt( rowSelected, 1).toString();
            alterAuditTableName.setText( selectedAuditTableName ); 
            if ( !alterAuditTableName.getText().isEmpty() ) {
                 alterAuditCheckBox.setSelected(true); 
                 alterAuditTableName.setForeground(Color.black);
            }
            
            //Set Trigger Names and check boxes
            alterBDTriggerName.setText( dataTable.getValueAt( rowSelected, 2).toString() ); 
            if ( !alterBDTriggerName.getText().isEmpty() ) {
                 alterBDCheckBox.setSelected(true); 
                 alterBDTriggerName.setForeground(Color.black);
            }
            
            alterBITriggerName.setText( dataTable.getValueAt( rowSelected, 3).toString() ); 
            if ( !alterBITriggerName.getText().isEmpty() ) {
                 alterBICheckBox.setSelected(true); 
                 alterBITriggerName.setForeground(Color.black);                
            }
            
            alterBUTriggerName.setText( dataTable.getValueAt( rowSelected, 4).toString() ); 
            if ( !alterBUTriggerName.getText().isEmpty() ) {
                 alterBUCheckBox.setSelected(true); 
                 alterBUTriggerName.setForeground(Color.black);  
            }
            
            alterADTriggerName.setText( dataTable.getValueAt( rowSelected, 5).toString() ); 
            if ( !alterADTriggerName.getText().isEmpty() ) {
                 alterADCheckBox.setSelected(true); 
                 alterADTriggerName.setForeground(Color.black);  
            }
            
            alterAITriggerName.setText( dataTable.getValueAt( rowSelected, 6).toString() ); 
            if ( !alterAITriggerName.getText().isEmpty() ) {
                 alterAICheckBox.setSelected(true); 
                 alterAITriggerName.setForeground(Color.black);   
            }
            
            alterAUTriggerName.setText( dataTable.getValueAt( rowSelected, 7).toString() ); 
            if ( !alterAUTriggerName.getText().isEmpty() ) {
                 alterAUCheckBox.setSelected(true); 
                 alterAUTriggerName.setForeground(Color.black);  
            }          
    }
    
    private void loadAlterFieldsMulti( int selectedIndexes[] ) {
        int rowSelected; 
        
        multiBDExists=false; multiBIExists=false; multiBUExists=false; 
        multiADExists=false; multiAIExists=false; multiAUExists=false; multiAuditExists=false;
        //Determine if any objects of each type exist for selected base tables. 
        for (int i=0; i<numSelected; i++) {

            rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
            //rowSelected = selectedIndexes[i];
            //System.out.println( "i: " + i + "   Table Index: " + rowSelected + "    View Index:  " + selectedIndexes[i] );          
            //selectedTableName = dataTable.getValueAt( rowSelected, 0).toString(); 
            
            if ( !dataTable.getValueAt( rowSelected, 1).toString().isEmpty()) multiAuditExists=true;
            
            if ( !dataTable.getValueAt( rowSelected, 2).toString().isEmpty()) multiBDExists=true;            
            if ( !dataTable.getValueAt( rowSelected, 3).toString().isEmpty()) multiBIExists=true; 
            if ( !dataTable.getValueAt( rowSelected, 4).toString().isEmpty()) multiBUExists=true; 
            if ( !dataTable.getValueAt( rowSelected, 5).toString().isEmpty()) multiADExists=true; 
            if ( !dataTable.getValueAt( rowSelected, 6).toString().isEmpty()) multiAIExists=true; 
            if ( !dataTable.getValueAt( rowSelected, 7).toString().isEmpty()) multiAUExists=true;          
        }
        
        //Now set Alter field values based on options and existence/nonexistence. 
        alterTableName.setText( multiStr ); 
        
        if ( multiRebuildAuditDefault ) {
            alterAuditCheckBox.setSelected(true);
            alterAuditTableName.setText(multiStr);
            if ( multiAuditExists )
                alterAuditTableName.setForeground(Color.black);  
        }
        
        if ( multiBDDefault ) {
            alterBDCheckBox.setSelected(true);
            alterBDTriggerName.setText(multiStr);
            if ( multiBDExists )
                alterBDTriggerName.setForeground(Color.black);  
        }
        
        if ( multiBIDefault ) {
            alterBICheckBox.setSelected(true);
            alterBITriggerName.setText(multiStr);
            if ( multiBIExists )
                alterBITriggerName.setForeground(Color.black);  
        }
                
        if ( multiBUDefault ) {
            alterBUCheckBox.setSelected(true);
            alterBUTriggerName.setText(multiStr);
            if ( multiBUExists )
                alterBUTriggerName.setForeground(Color.black);  
        }
        
        if ( multiADDefault ) {
            alterADCheckBox.setSelected(true);
            alterADTriggerName.setText(multiStr);
            if ( multiADExists )
                alterADTriggerName.setForeground(Color.black);  
        }
        
        if ( multiAIDefault ) {
            alterAICheckBox.setSelected(true);
            alterAITriggerName.setText(multiStr);
            if ( multiAIExists )
                alterAITriggerName.setForeground(Color.black);  
        }
                
        if ( multiAUDefault ) {
            alterAUCheckBox.setSelected(true);
            alterAUTriggerName.setText(multiStr);
            if ( multiAUExists )
                alterAUTriggerName.setForeground(Color.black);  
        }        
    }
    
    private void alterExitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterExitButtonActionPerformed
        
        alterFrame.setVisible(false);
        loadTableInfo(); 
        
    }//GEN-LAST:event_alterExitButtonActionPerformed

    private void alterDropAuditTableButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterDropAuditTableButtonActionPerformed
        int rowSelected; 
        
        // If Audit Table exists and is selected and name is not blank, drop it...
        if ( alterAuditTableName.getForeground() == Color.black & alterAuditCheckBox.isSelected() == true & !alterAuditTableName.getText().isEmpty() ) {
            if ( numSelected == 1 )
                dropAuditTable( alterAuditTableName.getText(), auditSchemaName ); 
            else if ( multiAuditExists ) {
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                    if ( !dataTable.getValueAt( rowSelected, 1).toString().isEmpty())  //Only drop existing tables. 
                        dropAuditTable( dataTable.getValueAt( rowSelected, 1).toString(), auditSchemaName );                  
                }                
                multiAuditExists = false; 
            }
        }
        loadTableInfo(); 
    }//GEN-LAST:event_alterDropAuditTableButtonActionPerformed

    private void alterCreateAuditTableButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterCreateAuditTableButtonActionPerformed
        int rowSelected; 
        // If Audit Table does not exist and is selected and name is not blank, create it...
        if (  alterAuditCheckBox.isSelected() == true & !alterAuditTableName.getText().isEmpty() ) {
            if ( numSelected == 1 & alterAuditTableName.getForeground() == Color.red )  //Table must not exist for single creation...
                createAuditTable( alterTableName.getText(), alterAuditTableName.getText(), schemaName, auditSchemaName ); 
            else  //...but for multi, we only create non-existant tables out of list with possible existing tables...
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                    selectedTableName = dataTable.getValueAt( rowSelected, 0).toString(); 
                    if ( dataTable.getValueAt( rowSelected, 1).toString().isEmpty())  //....and we don't overwrite existing tables. 
                        createAuditTable( selectedTableName, generateAuditTableName( selectedTableName ), schemaName, auditSchemaName );                   
                }  
        }
        loadTableInfo(); 
    }//GEN-LAST:event_alterCreateAuditTableButtonActionPerformed

    private void alterDropTriggersButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterDropTriggersButtonActionPerformed
        int rowSelected; 
        
        // If Trigger exists and is selected and name is not blank, drop it...
        if ( alterBDTriggerName.getForeground() == Color.black & alterBDCheckBox.isSelected() == true & !alterBDTriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 )
                dropTrigger( alterBDTriggerName, alterBDCheckBox ); 
            else if ( multiBDExists ) {
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);                
                    if ( !dataTable.getValueAt( rowSelected, 2).toString().isEmpty()) {  //I don't like this....
                        alterBDTriggerName.setText(dataTable.getValueAt( rowSelected, 2).toString());
                        dropTrigger( alterBDTriggerName, alterBDCheckBox ); 
                    }
                }
                multiBDExists = false; 
            }
        }
        
        // If Trigger exists and is selected and name is not blank, drop it...
        if ( alterBITriggerName.getForeground() == Color.black & alterBICheckBox.isSelected() == true & !alterBITriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 )
                dropTrigger( alterBITriggerName, alterBICheckBox ); 
            else if ( multiBIExists ) {
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);                
                    if ( !dataTable.getValueAt( rowSelected, 3).toString().isEmpty()) {  //I don't like this....
                        alterBITriggerName.setText(dataTable.getValueAt( rowSelected, 3).toString());
                        dropTrigger( alterBITriggerName, alterBICheckBox ); 
                    }
                }
                multiBIExists = false; 
            }            
        }
        
        // If Trigger exists and is selected and name is not blank, drop it...
        if ( alterBUTriggerName.getForeground() == Color.black & alterBUCheckBox.isSelected() == true & !alterBUTriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 ) 
                dropTrigger( alterBUTriggerName, alterBUCheckBox );
            else if ( multiBUExists ) {
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);                
                    if ( !dataTable.getValueAt( rowSelected, 4).toString().isEmpty()) {  //I don't like this....
                        alterBUTriggerName.setText(dataTable.getValueAt( rowSelected, 4).toString());
                        dropTrigger( alterBUTriggerName, alterBUCheckBox ); 
                    }
                }
                multiBUExists = false; 
            }            
        }
        
        // If Trigger exists and is selected and name is not blank, drop it...
        if ( alterADTriggerName.getForeground() == Color.black & alterADCheckBox.isSelected() == true & !alterADTriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 ) 
                dropTrigger( alterADTriggerName, alterADCheckBox ); 
            else if ( multiADExists ) {
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);                
                    if ( !dataTable.getValueAt( rowSelected, 5).toString().isEmpty()) {  //I don't like this....
                        alterADTriggerName.setText(dataTable.getValueAt( rowSelected, 5).toString());
                        dropTrigger( alterADTriggerName, alterADCheckBox ); 
                    }
                }
                multiADExists = false; 
            }              
        }
            
        // If Trigger exists and is selected and name is not blank, drop it...
        if ( alterAITriggerName.getForeground() == Color.black & alterAICheckBox.isSelected() == true & !alterAITriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 ) 
                dropTrigger( alterAITriggerName, alterAICheckBox ); 
            else if ( multiAIExists ) {
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);                
                    if ( !dataTable.getValueAt( rowSelected, 6).toString().isEmpty()) {  //I don't like this....
                        alterAITriggerName.setText(dataTable.getValueAt( rowSelected, 6).toString());
                        dropTrigger( alterAITriggerName, alterAICheckBox ); 
                    }
                }
                multiAIExists = false; 
            }              
        }
        
         // If Trigger exists and is selected and name is not blank, drop it...
        if ( alterAUTriggerName.getForeground() == Color.black & alterAUCheckBox.isSelected() == true & !alterAUTriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 ) 
                dropTrigger( alterAUTriggerName, alterAUCheckBox ); 
            else if ( multiAUExists ) {
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);                
                    if ( !dataTable.getValueAt( rowSelected, 7).toString().isEmpty()) {  //I don't like this....
                        alterAUTriggerName.setText(dataTable.getValueAt( rowSelected, 7).toString());
                        dropTrigger( alterAUTriggerName, alterAUCheckBox ); 
                    }
                }
                multiAUExists = false; 
            }  
        }
        
        loadTableInfo(); 
    }//GEN-LAST:event_alterDropTriggersButtonActionPerformed

    private void alterCreateTriggersButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterCreateTriggersButtonActionPerformed
        int rowSelected; 
                
        // If Trigger does not exist and is selected and name is not blank, create it...
        if ( alterBDCheckBox.isSelected() == true & !alterBDTriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 & alterBDTriggerName.getForeground() == Color.red )
                createTrigger( alterTableName.getText(), alterAuditTableName.getText(), alterBDTriggerName, schemaName, auditSchemaName, 
                                    alterBDCheckBox ); 
            else
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                    selectedTableName = dataTable.getValueAt( rowSelected, 0).toString();  //Necessary downstream....BAD
                    if ( dataTable.getValueAt( rowSelected, 2).toString().isEmpty()) {  //....Don't overwrite existing triggers. 
                        alterBDTriggerName.setText(generateBDTriggerName()); 
                        createTrigger( selectedTableName, generateAuditTableName( selectedTableName ), alterBDTriggerName, schemaName, 
                                            auditSchemaName, alterBDCheckBox ); 
                        alterBDTriggerName.setText(multiStr); 
                    }
                }
                multiBDExists = true; 
        }
        
        // If Trigger does not exist and is selected and name is not blank, create it...
        if ( alterBICheckBox.isSelected() == true & !alterBITriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 & alterBITriggerName.getForeground() == Color.red ) 
                createTrigger( alterTableName.getText(), alterAuditTableName.getText(), alterBITriggerName, schemaName, auditSchemaName, 
                                    alterBICheckBox ); 
            else
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                    selectedTableName = dataTable.getValueAt( rowSelected, 0).toString();  //Necessary downstream....BAD
                    if ( dataTable.getValueAt( rowSelected, 3).toString().isEmpty()) {  //....Don't overwrite existing triggers. 
                        alterBITriggerName.setText(generateBITriggerName()); 
                        createTrigger( selectedTableName, generateAuditTableName( selectedTableName ), alterBITriggerName, schemaName, 
                                            auditSchemaName, alterBICheckBox ); 
                        alterBITriggerName.setText(multiStr); 
                    }
                }
                multiBIExists = true; 
        }
        
        // If Trigger does not exist and is selected and name is not blank, create it...
        if ( alterBUCheckBox.isSelected() == true & !alterBUTriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 & alterBUTriggerName.getForeground() == Color.red )
                createTrigger( alterTableName.getText(), alterAuditTableName.getText(), alterBUTriggerName, schemaName, auditSchemaName, 
                                    alterBUCheckBox ); 
            else
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                    selectedTableName = dataTable.getValueAt( rowSelected, 0).toString();  //Necessary downstream....BAD
                    if ( dataTable.getValueAt( rowSelected, 4).toString().isEmpty()) {  //....Don't overwrite existing triggers. 
                        alterBUTriggerName.setText(generateBUTriggerName()); 
                        createTrigger( selectedTableName, generateAuditTableName( selectedTableName ), alterBUTriggerName, schemaName, 
                                            auditSchemaName, alterBUCheckBox ); 
                        alterBUTriggerName.setText(multiStr); 
                    }
                }
                multiBUExists = true; 
        }
        
        // If Trigger does not exist and is selected and name is not blank, create it...
        if ( alterADCheckBox.isSelected() == true & !alterADTriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 & alterADTriggerName.getForeground() == Color.red )
                createTrigger( alterTableName.getText(), alterAuditTableName.getText(), alterADTriggerName, schemaName, auditSchemaName, 
                                    alterADCheckBox ); 
            else
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                    selectedTableName = dataTable.getValueAt( rowSelected, 0).toString();  //Necessary downstream....BAD
                    if ( dataTable.getValueAt( rowSelected, 5).toString().isEmpty()) {  //....Don't overwrite existing triggers. 
                        alterADTriggerName.setText(generateADTriggerName()); 
                        createTrigger( selectedTableName, generateAuditTableName( selectedTableName ), alterADTriggerName, schemaName, 
                                            auditSchemaName, alterADCheckBox ); 
                        alterADTriggerName.setText(multiStr); 
                    }
                }
                multiADExists = true; 
        }
        
        // If Trigger does not exist and is selected and name is not blank, create it...
        if ( alterAICheckBox.isSelected() == true & !alterAITriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 & alterAITriggerName.getForeground() == Color.red ) 
                createTrigger( alterTableName.getText(), alterAuditTableName.getText(), alterAITriggerName, schemaName, auditSchemaName, 
                                    alterAICheckBox ); 
             else
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                    selectedTableName = dataTable.getValueAt( rowSelected, 0).toString();  //Necessary downstream....BAD
                    if ( dataTable.getValueAt( rowSelected, 6).toString().isEmpty()) {  //....Don't overwrite existing triggers. 
                        alterAITriggerName.setText(generateAITriggerName()); 
                        createTrigger( selectedTableName, generateAuditTableName( selectedTableName ), alterAITriggerName, schemaName, 
                                            auditSchemaName, alterAICheckBox ); 
                        alterAITriggerName.setText(multiStr); 
                    }
                }
                multiAIExists = true; 
        }
        
        // If Trigger does not exist and is selected and name is not blank, create it...
        if ( alterAUCheckBox.isSelected() == true & !alterAUTriggerName.getText().isEmpty() ) {
            if ( numSelected == 1 & alterAUTriggerName.getForeground() == Color.red ) 
                createTrigger( alterTableName.getText(), alterAuditTableName.getText(), alterAUTriggerName, schemaName, auditSchemaName, 
                                    alterAUCheckBox ); 
            else
                for (int i=0; i<numSelected; i++) {
                    rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                    selectedTableName = dataTable.getValueAt( rowSelected, 0).toString();  //Necessary downstream....BAD
                    if ( dataTable.getValueAt( rowSelected, 7).toString().isEmpty()) {  //....Don't overwrite existing triggers. 
                        alterAUTriggerName.setText(generateAUTriggerName()); 
                        createTrigger( selectedTableName, generateAuditTableName( selectedTableName ), alterAUTriggerName, schemaName, 
                                            auditSchemaName, alterAUCheckBox ); 
                        alterAUTriggerName.setText(multiStr); 
                    }
                }
                multiAUExists = true; 
        }
        
        loadTableInfo(); 

    }//GEN-LAST:event_alterCreateTriggersButtonActionPerformed

    private void alterAuditCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterAuditCheckBoxActionPerformed

        if ( alterAuditCheckBox.isSelected() ) {    //Generate        
            if ( alterAuditTableName.getForeground() == Color.red ) {
                if ( numSelected == 1 )
                    alterAuditTableName.setText(generateAuditTableName( selectedTableName ));
                else
                    alterAuditTableName.setText(multiStr);
                alterAuditCheckBox.setSelected(true);
            }              
        }
        else {  //Clear     
            if ( alterAuditTableName.getForeground() == Color.red ) {
                alterAuditTableName.setText(""); 
                alterAuditCheckBox.setSelected(false);
            }              
        }
    }//GEN-LAST:event_alterAuditCheckBoxActionPerformed

    private void optionsButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_optionsButtonActionPerformed

        //Load options for this schema into the Options frame
        optionsSchemaName.setText(schemaName); 
        optionsAuditPrefix.setText(auditTablePrefix); 
        optionsAuditSuffix.setText(auditTableSuffix); 
        optionsTmpAuditPrefix.setText(tmpAuditTablePrefix); 
        optionsTriggerNameFormatString.setText(triggerNameFormat); 
        optionsBeforeString.setText(beforeString); 
        optionsAfterString.setText(afterString); 
        optionsDeleteString.setText(deleteString); 
        optionsInsertString.setText(insertString); 
        optionsUpdateString.setText(updateString);  
        optionsBDCheckBox.setSelected(multiBDDefault); 
        optionsBICheckBox.setSelected(multiBIDefault); 
        optionsBUCheckBox.setSelected(multiBUDefault); 
        optionsADCheckBox.setSelected(multiADDefault); 
        optionsAICheckBox.setSelected(multiAIDefault); 
        optionsAUCheckBox.setSelected(multiAUDefault); 
        optionsSaveRestoreCheckBox.setSelected(multiSaveRestoreDefault); 
        optionsRebuildAuditCheckBox.setSelected(multiRebuildAuditDefault); 
        
        loadSchemaNames( optionsAuditSchemaComboBox ); 
        
        // Find current Audit Schema in Combo Box
        for( int i=0; i<optionsAuditSchemaComboBox.getItemCount(); i++ ){
            if (optionsAuditSchemaComboBox.getItemAt(i).equals(auditSchemaName)) {
                optionsAuditSchemaComboBox.setSelectedIndex(i); 
                break;
            }
        }
               
        optionsFrame.setVisible(true); 
    }//GEN-LAST:event_optionsButtonActionPerformed

    private void optionsSaveButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_optionsSaveButtonActionPerformed

        // Save Options Frame info back to global variables, write back to database. 

        auditTablePrefix = optionsAuditPrefix.getText(); 
        auditTableSuffix = optionsAuditSuffix.getText(); 
        tmpAuditTablePrefix = optionsTmpAuditPrefix.getText(); 
        triggerNameFormat = optionsTriggerNameFormatString.getText(); 
        beforeString = optionsBeforeString.getText(); 
        afterString = optionsAfterString.getText(); 
        deleteString = optionsDeleteString.getText(); 
        insertString = optionsInsertString.getText(); 
        updateString = optionsUpdateString.getText();
        multiBDDefault = optionsBDCheckBox.isSelected(); 
        multiBIDefault = optionsBICheckBox.isSelected(); 
        multiBUDefault = optionsBUCheckBox.isSelected(); 
        multiADDefault = optionsADCheckBox.isSelected(); 
        multiAIDefault = optionsAICheckBox.isSelected(); 
        multiAUDefault = optionsAUCheckBox.isSelected(); 
        multiSaveRestoreDefault = optionsSaveRestoreCheckBox.isSelected(); 
        multiRebuildAuditDefault = optionsRebuildAuditCheckBox.isSelected(); 
        
        auditSchemaName = optionsAuditSchemaComboBox.getSelectedItem().toString(); 
        
        saveTManInfo(); 
        loadTManInfo(); 
        loadTableInfo(); 
        
        optionsFrame.setVisible(false); 
    }//GEN-LAST:event_optionsSaveButtonActionPerformed

    private void optionsExitButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_optionsExitButtonActionPerformed

        // Don't save any changes...       
        optionsFrame.setVisible(false); 
    }//GEN-LAST:event_optionsExitButtonActionPerformed

    private void optionsRebuildAuditCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_optionsRebuildAuditCheckBoxActionPerformed
        if ( optionsRebuildAuditCheckBox.isSelected() ) {
            optionsSaveRestoreCheckBox.setEnabled(true); 
            optionsSaveRestoreCheckBox.setSelected(true); 
        }
        else {
            optionsSaveRestoreCheckBox.setEnabled(false);    
            optionsSaveRestoreCheckBox.setSelected(false); 
        }
    }//GEN-LAST:event_optionsRebuildAuditCheckBoxActionPerformed

    private void aboutFrameMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_aboutFrameMouseClicked
        aboutFrame.setVisible(false);
    }//GEN-LAST:event_aboutFrameMouseClicked

    private void aboutButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutButtonActionPerformed
        aboutFrame.setVisible(true);
    }//GEN-LAST:event_aboutButtonActionPerformed

    private void alterBDCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterBDCheckBoxActionPerformed
        if ( alterBDCheckBox.isSelected() ) {  //Generate
            if ( alterBDTriggerName.getForeground() == Color.red ) {
                if ( numSelected == 1 )
                    alterBDTriggerName.setText(generateBDTriggerName()); 
                else
                alterBDTriggerName.setText(multiStr); 
            }             
        }
        else {  //Clear
            if ( alterBDTriggerName.getForeground() == Color.red ) {
                    alterBDTriggerName.setText(""); 
            }          
        }
    }//GEN-LAST:event_alterBDCheckBoxActionPerformed

    private void alterBICheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterBICheckBoxActionPerformed
        if ( alterBICheckBox.isSelected() ) {  //Generate
            if ( alterBITriggerName.getForeground() == Color.red ) {
                if ( numSelected == 1 )
                    alterBITriggerName.setText(generateBITriggerName()); 
                else
                alterBITriggerName.setText(multiStr); 
            }             
        }
        else {  //Clear
            if ( alterBITriggerName.getForeground() == Color.red ) {
                    alterBITriggerName.setText(""); 
            }          
        }
    }//GEN-LAST:event_alterBICheckBoxActionPerformed

    private void alterBUCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterBUCheckBoxActionPerformed
        if ( alterBUCheckBox.isSelected() ) {  //Generate
            if ( alterBUTriggerName.getForeground() == Color.red ) {
                if ( numSelected == 1 )
                    alterBUTriggerName.setText(generateBUTriggerName()); 
                else
                alterBUTriggerName.setText(multiStr); 
            }             
        }
        else {  //Clear
            if ( alterBUTriggerName.getForeground() == Color.red ) {
                    alterBUTriggerName.setText(""); 
            }          
        }
    }//GEN-LAST:event_alterBUCheckBoxActionPerformed

    private void alterADCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterADCheckBoxActionPerformed
        if ( alterADCheckBox.isSelected() ) {  //Generate
            if ( alterADTriggerName.getForeground() == Color.red ) {
                if ( numSelected == 1 )
                    alterADTriggerName.setText(generateADTriggerName()); 
                else
                alterADTriggerName.setText(multiStr); 
            }             
        }
        else {  //Clear
            if ( alterADTriggerName.getForeground() == Color.red ) {
                    alterADTriggerName.setText(""); 
            }          
        }
    }//GEN-LAST:event_alterADCheckBoxActionPerformed

    private void alterAICheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterAICheckBoxActionPerformed
        if ( alterAICheckBox.isSelected() ) {  //Generate
            if ( alterAITriggerName.getForeground() == Color.red ) {
                if ( numSelected == 1 )
                    alterAITriggerName.setText(generateAITriggerName()); 
                else
                alterAITriggerName.setText(multiStr); 
            }             
        }
        else {  //Clear
            if ( alterAITriggerName.getForeground() == Color.red ) {
                    alterAITriggerName.setText(""); 
            }          
        }
    }//GEN-LAST:event_alterAICheckBoxActionPerformed

    private void alterAUCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_alterAUCheckBoxActionPerformed
        if ( alterAUCheckBox.isSelected() ) {  //Generate
            if ( alterAUTriggerName.getForeground() == Color.red ) {
                if ( numSelected == 1 )
                    alterAUTriggerName.setText(generateAUTriggerName()); 
                else
                alterAUTriggerName.setText(multiStr); 
            }             
        }
        else {  //Clear
            if ( alterAUTriggerName.getForeground() == Color.red ) {
                    alterAUTriggerName.setText(""); 
            }          
        }
    }//GEN-LAST:event_alterAUCheckBoxActionPerformed
    
    
    public void initConnectFields() {
        
        statusTextField.setText("Disconnected"); 
        schemaTextField.setText(""); 
        auditSchemaTextField.setText(""); 
        
        connectButton.setEnabled(true);
        disconnectButton.setEnabled(false); 
        alterButton.setEnabled(false);
        optionsButton.setEnabled(false);
        connectComboBox.removeAllItems();
        
    }
    
    
    public void initAlterFields() {
        
        alterTableName.setText( "" ); 
        alterAuditTableName.setText( "" );
        alterBDTriggerName.setText( "" ); 
        alterBITriggerName.setText( "" ); 
        alterBUTriggerName.setText( "" ); 
        alterADTriggerName.setText( "" ); 
        alterAITriggerName.setText( "" ); 
        alterAUTriggerName.setText( "" ); 
        
        //Assume non-existance
        alterAuditTableName.setForeground(Color.red);
        alterBDTriggerName.setForeground(Color.red);
        alterBITriggerName.setForeground(Color.red);
        alterBUTriggerName.setForeground(Color.red);
        alterADTriggerName.setForeground(Color.red);
        alterAITriggerName.setForeground(Color.red);
        alterAUTriggerName.setForeground(Color.red);

        alterAuditCheckBox.setSelected(false);
        alterBDCheckBox.setSelected(false);
        alterBICheckBox.setSelected(false);
        alterBUCheckBox.setSelected(false);
        alterADCheckBox.setSelected(false);
        alterAICheckBox.setSelected(false);
        alterAUCheckBox.setSelected(false);
    }
    
    
    public void loadTableInfo() {
        
        //Load information on tables in dbName. 
        //System.out.println( "Load Table Data" ); 
        loadInProgress = true; 
        try {

            // Initialize Table
            String sqlSelectTbl = "call triggerman.getAuditData('"+schemaName+"', '"+auditSchemaName+"')";
            String sqlMatchTbl = "SELECT triggerman.compareAuditTbl(?, ?, ?, ?) FROM dual"; 
            PreparedStatement ps; 
            Statement stmt;
            ResultSet rs;
            int rowSelected;  
            RowSorter<? extends TableModel> rSort; 
            List<? extends SortKey> rList; 
            TableModel myModel; 
            boolean match; 
            
            //System.out.print("Pre Num: "+numSelected); 
            numSelected = dataTable.getSelectedRowCount(); 
            selectedViewIndexes = dataTable.getSelectedRows();
            for (int i=0; i<numSelected; i++) {
               selectedIndexes[i] = dataTable.convertRowIndexToModel(selectedViewIndexes[i]);                 
            }

            saveColumnWidths( dataTable ); 

            rSort = dataTable.getRowSorter(); 
            rList = rSort.getSortKeys(); 
            
            //System.out.println(" Post Num: "+numSelected); 
            stmt = infoConn.createStatement( ResultSet.TYPE_SCROLL_INSENSITIVE,
                                             ResultSet.CONCUR_READ_ONLY );

            rs = stmt.executeQuery(sqlSelectTbl);

            ResultSetTableModel rstm =
                    new ResultSetTableModel(rs);

            dataTable.setModel(rstm);
            
            // Set Header Style
            JTableHeader h = dataTable.getTableHeader();
            h.setFont(dataTable.getFont());
            h.setForeground(Color.darkGray);
            h.setBackground(Color.lightGray);
            
            
            //Reset Selected Rows
            for (int i=0; i<numSelected; i++) {
                rowSelected = dataTable.convertRowIndexToView(selectedIndexes[i]);
                //rowSelected = selectedIndexes[i];
                dataTable.addRowSelectionInterval(rowSelected, rowSelected);
            }
           
            rSort = dataTable.getRowSorter(); 
            rSort.setSortKeys(rList);
           
            restoreColumnWidths( dataTable ); 
                  
            loadInProgress = false; 
        }
        catch (Exception e) {
            e.printStackTrace( System.err );
            JOptionPane.showMessageDialog(this,
            "Load Table Info: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);          
        }          
    }
    
    
    public void saveColumnWidths( JTable t ) {
        widthSaved = false; 
        int colNum = t.getColumnModel().getColumnCount();   
        //System.out.println("Save colNum: "+colNum); 
        for ( int i=0; i<colNum; i++) {
            //System.out.println("Save  i: "+i+"  width: "+t.getColumnModel().getColumn( i ).getWidth() ); 
            columnWidths[i] = t.getColumnModel().getColumn( i ).getWidth(); 
            widthSaved = true; 
        }
     }
    

    public void restoreColumnWidths( JTable t ) {
        
        int colNum = t.getColumnModel().getColumnCount();   
        //System.out.println("Restore colNum: "+colNum); 
        if ( widthSaved ) {
         for ( int i=0; i<colNum; i++) {
            //t.getColumnModel().getColumn( i ).setMaxWidth( columnWidths[i] );   
            //t.getColumnModel().getColumn( i ).setMinWidth( columnWidths[i] ); 
            //t.getColumnModel().getColumn( i ).setPreferredWidth( columnWidths[i] ); 
            t.getColumnModel().getColumn( i ).setPreferredWidth( columnWidths[i] ); 
            //System.out.println("Restore  i: "+i+"  width: "+columnWidths[i] );             
         }
         widthSaved = false; 
         
        }
     }    
    
    
    public void loadTManInfo() {
        
        //Load information from tman_options table. 
   
        try {

            // Initialize Table
            String sqlSelectInfo = "call triggerman.getTmanInfo('"+schemaName+"')";
            String sqlDefaultInfo = "call triggerman.setTmanDefault('"+schemaName+"')";
            PreparedStatement stmt;
            ResultSet rs;

            // Get Tman_Options info for this schema. 
            stmt = infoConn.prepareStatement(sqlSelectInfo);
            rs = stmt.executeQuery(); 
            rs.next(); 
            
            if ( rs.getRow() == 0 ) {
                // Schema not found, ask not nicely, create default. 
                
                JOptionPane.showMessageDialog(this,
                "Schema not Initialized \n Default Options values will be created for: " + schemaName,
                null, JOptionPane.WARNING_MESSAGE);                
                
                stmt = infoConn.prepareStatement(sqlDefaultInfo);     
                stmt.executeQuery(); 
                
                // Try, Try Again...
                stmt = infoConn.prepareStatement(sqlSelectInfo);                            
                rs = stmt.executeQuery(); 
                rs.next();   
            }

            auditTablePrefix = rs.getString("audit_table_prefix"); 
            auditTableSuffix = rs.getString("audit_table_suffix"); 
            triggerNameFormat = rs.getString("trigger_name_format"); 
            beforeString = rs.getString("before_string"); 
            afterString = rs.getString("after_string"); 
            deleteString = rs.getString("delete_string"); 
            insertString = rs.getString("insert_string"); 
            updateString = rs.getString("update_string"); 
            auditSchemaName = rs.getString("audit_schema_name");
            tmpAuditTablePrefix = rs.getString("tmp_audit_table_prefix"); 
            multiBDDefault = rs.getBoolean("auto_BD"); 
            multiBIDefault = rs.getBoolean("auto_BI"); 
            multiBUDefault = rs.getBoolean("auto_BU"); 
            multiADDefault = rs.getBoolean("auto_AD"); 
            multiAIDefault = rs.getBoolean("auto_AI"); 
            multiAUDefault = rs.getBoolean("auto_AU"); 
            multiSaveRestoreDefault = rs.getBoolean("auto_save_restore"); 
            multiRebuildAuditDefault = rs.getBoolean("auto_rebuild_audit"); 
            
            if ( auditSchemaName.isEmpty() ) {
                auditSchemaName = schemaName; 
            }
            //System.out.println("Audit Schema: "+auditSchemaName); 
            auditSchemaTextField.setText( auditSchemaName );

        }
        catch (Exception e) {
           JOptionPane.showMessageDialog(this,
            "Load TMan Info: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
        }          
    }
    
    
    public void saveTManInfo() {
        
        // Save info back into the TMan_Options table. 
        
        PreparedStatement insertTManInfo;

        String insertStmt = "call triggerman.saveTmanInfo( ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?,?,?,?,?,?,?,?, ? )";
                        
        try {
                conn.setAutoCommit(false);
                insertTManInfo = conn.prepareStatement(insertStmt);

                insertTManInfo.setString(1, schemaName);
                insertTManInfo.setString(2, auditTablePrefix);
                insertTManInfo.setString(3, auditTableSuffix);
                insertTManInfo.setString(4, triggerNameFormat);
                insertTManInfo.setString(5, beforeString);
                insertTManInfo.setString(6, afterString);
                insertTManInfo.setString(7, deleteString);
                insertTManInfo.setString(8, insertString);
                insertTManInfo.setString(9, updateString);
                insertTManInfo.setString(10, auditSchemaName);
                insertTManInfo.setString(11, tmpAuditTablePrefix);
                insertTManInfo.setBoolean(12, multiBDDefault); 
                insertTManInfo.setBoolean(13, multiBIDefault); 
                insertTManInfo.setBoolean(14, multiBUDefault); 
                insertTManInfo.setBoolean(15, multiADDefault); 
                insertTManInfo.setBoolean(16, multiAIDefault); 
                insertTManInfo.setBoolean(17, multiAUDefault); 
                insertTManInfo.setBoolean(18, multiSaveRestoreDefault); 
                insertTManInfo.setBoolean(19, multiRebuildAuditDefault); 
                
                insertTManInfo.execute();
                conn.commit();
                insertTManInfo.close();       
                
        }
        catch (Exception e) {
           JOptionPane.showMessageDialog(this,
            "Save TMan Info: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
        }
    }
    
    
    private boolean checkTManInit() {      
        // This routine checks to determine if the TriggerMan 
        // schema and objects exist. If not, it gives the 
        // option of creating them. 
       try {
        if ( checkTManSchema() == 0 ) {
            
            JOptionPane.showMessageDialog(this,
            "TriggerMan does not seem to be installed. \n Please run the TriggerMan.sql script in MySQL", 
            null, JOptionPane.ERROR_MESSAGE ); 
            return(false);  
            /*
             if ( answer == JOptionPane.YES_OPTION ) {          

                 initTManSchema(); 
                 initTManTable(); 
                 initTManProcedures(); 
            } */
        }
        return(true); 
       } 
       catch (Exception e) {         
           // Something failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
           "checkTmanInit: " + e.getMessage(),
           null, JOptionPane.ERROR_MESSAGE);
           return(false); 
       } 
    }
    
    
    private int checkTManSchema() {
        //SQLNeeded
        String sqlSchemaExist = "SELECT COUNT(schema_name) FROM schemata WHERE schema_name = 'triggerman'";
        PreparedStatement schemaStmt;
        ResultSet schemaRS;
        
       try {
           
        // Check for TriggerMan schema.            
        schemaStmt = infoConn.prepareStatement(sqlSchemaExist);
        schemaRS = schemaStmt.executeQuery();
        schemaRS.next(); 
        return( schemaRS.getInt(1)); 
 
       } 
       catch (Exception e) {
           
           // Something failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "checkTManSchema: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
            return(0); 
       }      
    }    

    
    private void disconnectTman() {
        
        try {
            
            if ( infoConn != null )
                infoConn.close(); 
            
            if ( conn != null )
                conn.close(); 
            
            initConnectFields();  
                        
            dataTable.setModel(nullModel);   
            
            optionsButton.setEnabled(false); 
 
    	  }
        catch(Exception e) {           
           // Disconnect attempt failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "Disconnect: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);       
        } 
    }
    
  
    private String generateAuditTableName( String tableName ) {
        
        String auditTableName; 
        
        auditTableName = auditTablePrefix + tableName + auditTableSuffix;   
        
        //System.out.println( "Audit Table Name: " + auditTableName ); 
        return auditTableName; 
    }    
    
    private String generateTriggerName( String tableName, String timing, String event, String triggerFormat ) {
        
        String triggerName; 
        
        triggerName = triggerFormat.replaceAll("%TBL%", tableName ).replaceAll("%TIMING%", timing ).replaceAll("%EVENT%", event );  
        
        //System.out.println( "Trigger Name: " + triggerName ); 
        return triggerName; 
    }
    
    private String generateBDTriggerName( ) {
        return generateTriggerName( selectedTableName, beforeString, deleteString, triggerNameFormat ); 
    }
    
    private String generateBITriggerName( ) {
        return generateTriggerName( selectedTableName, beforeString, insertString, triggerNameFormat ); 
    }
    
    private String generateBUTriggerName( ) {
        return generateTriggerName( selectedTableName, beforeString, updateString, triggerNameFormat ); 
    }
    
    private String generateADTriggerName( ) {
        return generateTriggerName( selectedTableName, afterString, deleteString, triggerNameFormat ); 
    }
    
    private String generateAITriggerName( ) {
        return generateTriggerName( selectedTableName, afterString, insertString, triggerNameFormat ); 
    }
    
    private String generateAUTriggerName( ) {
        return generateTriggerName( selectedTableName, afterString, updateString, triggerNameFormat ); 
    }
    
    private void dropAuditTable( String auditTableName, String auditSchemaName ) {

        String dropTbl = "DROP TABLE IF EXISTS "; 
        String countTbl = "SELECT COUNT(*) FROM " + auditSchemaName + "." + auditTableName; 
        String dropTmpTbl = "DROP TABLE IF EXISTS " + auditSchemaName + "." + tmpAuditTablePrefix + auditTableName; 
        String createTmpTbl = "CREATE TABLE " + auditSchemaName + "." + tmpAuditTablePrefix + auditTableName +
                            " AS SELECT * FROM " + auditSchemaName + "." + auditTableName;
        PreparedStatement ps;
        ResultSet rs; 
        int count; 
        int answer = JOptionPane.NO_OPTION; 
        boolean dropped; 

       try { 
        // If Audit Table exists, get record count. 
        if ( alterAuditTableName.getForeground() == Color.black ) {
            ps = conn.prepareStatement( countTbl );
            rs = ps.executeQuery(); rs.next(); count = rs.getInt(1); 
            // If Audit Table has data, ask user if they'd like to save it. 
            if( count > 0 ) {
                if ( numSelected == 1 ) //Only ask question if single proc, use option for multi proc. 
                    answer = JOptionPane.showConfirmDialog( alterFrame, "Would you like to save " + count + " rows of Audit Table data?",
                                                                null, JOptionPane.YES_NO_OPTION);
                if ( answer == JOptionPane.YES_OPTION |
                        ( numSelected > 1 &  multiSaveRestoreDefault )) {
                    // Drop, Create Tmp Audit table. 
                    ps = conn.prepareStatement( dropTmpTbl ); 
                    ps.execute(); 
                    ps = conn.prepareStatement( createTmpTbl ); 
                    ps.execute(); 
                } 
            } 
        }
        // Drop Audit Table
        ps = conn.prepareStatement( dropTbl + auditSchemaName + "." + auditTableName );
        dropped = ps.execute();
        
        // If successful, clear Audit Table info
        alterAuditTableName.setText( "" ); 
        alterAuditTableName.setForeground(Color.red);
        alterAuditCheckBox.setSelected(false);
       }       
       catch (Exception e) {          
           // Something failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "dropAuditTable: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
       }         
    }
    
    private void createAuditTable( String tableName, String auditTableName, String schemaName, String auditSchemaName ) {
  
       try { 
           
        // Create the specified table
        String createStr;
        String restoreStr;
        String auditTblDef = "SELECT triggerman.getAuditTblDef( '" + tableName + "', '" + auditTableName + "', '" + 
                                                                     schemaName + "', '" + auditSchemaName + "' ) FROM dual"; 
        //SQLNeeded
        String existTmpAuditTbl = "SELECT COUNT(*) FROM information_schema.tables WHERE table_name = '" + tmpAuditTablePrefix + auditTableName +
                                    "' AND table_schema = '" + auditSchemaName + "';";
        String countTmpAuditTbl = "SELECT COUNT(*) FROM " + auditSchemaName + "." + tmpAuditTablePrefix + auditTableName; 
        String dropTmpTbl = "DROP TABLE IF EXISTS " + auditSchemaName + "." + tmpAuditTablePrefix + auditTableName; 
        String restoreTmpTbl = "SELECT triggerman.restoreTmpTable ( '" + auditTableName + "', '" + auditSchemaName + "', '" + tmpAuditTablePrefix + "' );"; 
        PreparedStatement ps;
        ResultSet rs;
        int count; 
        int answer = JOptionPane.NO_OPTION; 
        int tmpCount; 
   
        // Generate CREATE TABLE statement. 
        ps = infoConn.prepareStatement( auditTblDef );
        rs = ps.executeQuery(); rs.next(); 
        createStr = rs.getString(1); 
        // Create Audit Table. 
        //System.out.println( createStr ); 
        ps = conn.prepareStatement( createStr ); 
        ps.execute(); 
        
        // If successful, set Audit Table info to 'Exists'
        alterAuditTableName.setForeground(Color.black);
        alterAuditCheckBox.setSelected(false);   
        
        // If Tmp Audit Table exists, ask user if they'd like to restore data. 
        ps = infoConn.prepareStatement( existTmpAuditTbl ); 
        rs = ps.executeQuery(); rs.next(); count = rs.getInt(1); 
        if ( count == 1 ) {
            // Get Count, Restore From and Drop Tmp Audit table. 
            ps = infoConn.prepareStatement( countTmpAuditTbl ); 
            rs = ps.executeQuery(); rs.next(); tmpCount = rs.getInt(1);
            if ( numSelected == 1 ) //Only ask if single proc, use option for multi proc. 
                answer = JOptionPane.showConfirmDialog( alterFrame, "Would you like to restore " + tmpCount + " rows of saved Audit Table data?",
                                                        null, JOptionPane.YES_NO_OPTION);
            if ( answer == JOptionPane.YES_OPTION |
                    ( numSelected > 1 & multiSaveRestoreDefault )) { 
                // Generate Restore ( INSERT INTO...SELECT FROM ) statement. 
                ps = infoConn.prepareStatement( restoreTmpTbl );
                rs = ps.executeQuery(); rs.next(); 
                restoreStr = rs.getString(1);                
                //System.out.println( restoreStr ); 
                ps = conn.prepareStatement( restoreStr ); 
                ps.execute(); 
            } 
            
            ps = conn.prepareStatement( dropTmpTbl ); 
            ps.execute(); 
        }
       }       
       catch (Exception e) {          
           // Something failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "createAuditTable: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
       }         
    }    
    
    
    private void dropTrigger( JTextField trigger, JCheckBox box ) {
  
       try { 
           
        // Drop the specified trigger
        String dropTrg = "DROP TRIGGER IF EXISTS "; 
        PreparedStatement schemaStmt;
        boolean dropped; 
   
        schemaStmt = conn.prepareStatement( dropTrg + trigger.getText() );
        dropped = schemaStmt.execute();
        
        // If successful, clear Audit Table info
        if ( !dropped  ) {
            trigger.setText( "" ); 
            trigger.setForeground(Color.red);
            box.setSelected(false);
        }
        else {
           // Something failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "dropTrigger: Trigger Not Dropped?",
            null, JOptionPane.ERROR_MESSAGE);
        }
       }        
       catch (Exception e) {        
           // Something failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "dropTrigger: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
       }         
    }    
  
    private void createTrigger( String tableName, String auditTableName, JTextField trigger, String schemaName, String auditSchemaName, JCheckBox box ) {
  
       try { 
           
        // Create the specified trigger
        String createStr;
        String triggerDefPre = "SELECT triggerman.getTriggerDef( '" + tableName + "', '" + auditTableName + "', '" + 
                                trigger.getText() + "', '" + schemaName + "', '" + auditSchemaName + "', '";  
        String triggerDefPost = " ) FROM dual";   //Need Time and Event in between
        String triggerStr; 
        PreparedStatement ps;
        ResultSet rs;
        boolean created; 
        String timeStr = ""; 
        String eventStr = ""; 
        
        //System.out.println( "Create Trigger: " ); 
        //System.out.println( "Trigger Object Name: " + trigger.getName() ); 
        //System.out.println( "Trigger Object Text: " + trigger.getText() ); 
        
        if ( auditTableName.isEmpty() ) {   
            JOptionPane.showMessageDialog(this,
            "Cannot create Triggers without Audit Table",
            null, JOptionPane.ERROR_MESSAGE);   
            return; 
        }
       
        // Get object name to determine Time and Event: alter(T)(E)TriggerName
        if ( trigger.getName().charAt(5) == 'A' ) {
            timeStr = "AFTER";     
        }
        else if ( trigger.getName().charAt(5) == 'B' ) {
            timeStr = "BEFORE";     
        }
        else
            timeStr = "TimeError"; 

        if ( trigger.getName().charAt(6) == 'D' ) {
            eventStr = "DELETE";     
        }
        else if ( trigger.getName().charAt(6) == 'I' ) {
            eventStr = "INSERT";     
        }
        else if ( trigger.getName().charAt(6) == 'U' ) {
            eventStr = "UPDATE";     
        }
        else
            eventStr = "EventError";
                
        triggerStr = triggerDefPre + timeStr + "', '" + eventStr + "' " + triggerDefPost; 
        //System.out.println( "Trigger Function Str: " + triggerStr ); 
        ps = infoConn.prepareStatement( triggerStr );
        rs = ps.executeQuery();
        rs.next(); 
        createStr = rs.getString(1); 
        
        //System.out.println( "Trigger Create Str: " + createStr ); 
        ps = conn.prepareStatement( createStr ); 
        created = ps.execute(); 
        
        // If successful, set trigger field properties
        if ( !created  ) {
            //alterAuditTableName.setText( "" ); 
            trigger.setForeground(Color.black);
            box.setSelected(false);
        }
        else {
           // Something failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "createTrigger: Trigger Not Created?",
            null, JOptionPane.ERROR_MESSAGE);
        }      
       }       
       catch (Exception e) {   
           // Something failed, put up dialog box. 
           JOptionPane.showMessageDialog(this,
            "createTrigger: " + e.getMessage(),
            null, JOptionPane.ERROR_MESSAGE);
       }         
    } 
    
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /*
         * Set the Nimbus look and feel
         */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /*
         * If Nimbus (introduced in Java SE 6) is not available, stay with the
         * default look and feel. For details see
         * http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(TriggerMan.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(TriggerMan.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(TriggerMan.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(TriggerMan.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /*
         * Create and display the form
         */
        java.awt.EventQueue.invokeLater(new Runnable() {

            public void run() {
                new TriggerMan().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox OSICheckBox;
    private javax.swing.JButton aboutButton;
    private javax.swing.JFrame aboutFrame;
    private javax.swing.JTextPane aboutTextPane;
    private javax.swing.JCheckBox alterADCheckBox;
    private javax.swing.JTextField alterADTriggerName;
    private javax.swing.JLabel alterADTriggerNameLabel;
    private javax.swing.JCheckBox alterAICheckBox;
    private javax.swing.JTextField alterAITriggerName;
    private javax.swing.JLabel alterAITriggerNameLabel;
    private javax.swing.JCheckBox alterAUCheckBox;
    private javax.swing.JTextField alterAUTriggerName;
    private javax.swing.JLabel alterAUTriggerNameLabel;
    private javax.swing.JCheckBox alterAuditCheckBox;
    private javax.swing.JTextField alterAuditTableName;
    private javax.swing.JLabel alterAuditTableNameLabel;
    private javax.swing.JCheckBox alterBDCheckBox;
    private javax.swing.JTextField alterBDTriggerName;
    private javax.swing.JLabel alterBDTriggerNameLabel;
    private javax.swing.JCheckBox alterBICheckBox;
    private javax.swing.JTextField alterBITriggerName;
    private javax.swing.JLabel alterBITriggerNameLabel;
    private javax.swing.JCheckBox alterBUCheckBox;
    private javax.swing.JTextField alterBUTriggerName;
    private javax.swing.JLabel alterBUTriggerNameLabel;
    private javax.swing.JButton alterButton;
    private javax.swing.JButton alterCreateAuditTableButton;
    private javax.swing.JButton alterCreateTriggersButton;
    private javax.swing.JButton alterDropAuditTableButton;
    private javax.swing.JButton alterDropTriggersButton;
    private javax.swing.JButton alterExitButton;
    private javax.swing.JFrame alterFrame;
    private javax.swing.JTextField alterTableName;
    private javax.swing.JLabel alterTableNameLabel;
    private javax.swing.JLabel auditSchemaLabel;
    private javax.swing.JTextField auditSchemaTextField;
    private javax.swing.JButton connectButton;
    private javax.swing.JButton connectCancelButton;
    private javax.swing.JComboBox connectComboBox;
    private javax.swing.JButton connectConnectButton;
    private javax.swing.JFrame connectFrame;
    private javax.swing.JButton connectOKButton;
    private javax.swing.JTextField connectTextField;
    private javax.swing.JTextField connectURL;
    private javax.swing.JLabel connectURLLabel;
    private javax.swing.JTable dataTable;
    private javax.swing.JLabel databaseLabel;
    private javax.swing.JButton disconnectButton;
    private javax.swing.JCheckBox impiricalCheckBox;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JCheckBox optionsADCheckBox;
    private javax.swing.JCheckBox optionsAICheckBox;
    private javax.swing.JCheckBox optionsAUCheckBox;
    private javax.swing.JLabel optionsAfterLabel;
    private javax.swing.JTextField optionsAfterString;
    private javax.swing.JTextField optionsAuditPrefix;
    private javax.swing.JLabel optionsAuditPrefixLabel;
    private javax.swing.JComboBox optionsAuditSchemaComboBox;
    private javax.swing.JLabel optionsAuditSchemaLabel;
    private javax.swing.JTextField optionsAuditSuffix;
    private javax.swing.JLabel optionsAuditSuffixLabel;
    private javax.swing.JCheckBox optionsBDCheckBox;
    private javax.swing.JCheckBox optionsBICheckBox;
    private javax.swing.JCheckBox optionsBUCheckBox;
    private javax.swing.JLabel optionsBeforeLabel;
    private javax.swing.JTextField optionsBeforeString;
    private javax.swing.JButton optionsButton;
    private javax.swing.JLabel optionsDefaultTriggersLabel;
    private javax.swing.JLabel optionsDeleteLabel;
    private javax.swing.JTextField optionsDeleteString;
    private javax.swing.JButton optionsExitButton;
    private javax.swing.JFrame optionsFrame;
    private javax.swing.JLabel optionsHeaderLabel;
    private javax.swing.JLabel optionsInsertLabel;
    private javax.swing.JTextField optionsInsertString;
    private javax.swing.JLabel optionsMultiLabel;
    private javax.swing.JCheckBox optionsRebuildAuditCheckBox;
    private javax.swing.JButton optionsSaveButton;
    private javax.swing.JCheckBox optionsSaveRestoreCheckBox;
    private javax.swing.JTextField optionsSchemaName;
    private javax.swing.JLabel optionsSchemaNameLabel;
    private javax.swing.JTextField optionsTmpAuditPrefix;
    private javax.swing.JTextField optionsTriggerNameFormatString;
    private javax.swing.JLabel optionsUpdateLabel;
    private javax.swing.JTextField optionsUpdateString;
    private javax.swing.JLabel passwordLabel;
    private javax.swing.JPasswordField passwordPasswordField;
    private javax.swing.JTextField schemaTextField;
    private javax.swing.JLabel statusLabel;
    private javax.swing.JTextField statusTextField;
    private javax.swing.JLabel tmpAuditTablePrefixLabel;
    private javax.swing.JLabel usernameLabel;
    private javax.swing.JTextField usernameTextField;
    // End of variables declaration//GEN-END:variables
}
